;; flashcheck.asm
;; checking flash memory
;; Hardware: mk3 board with myAVRstamp256 (Atmega2560)
;;           connected LEDs at PORTL and PORTB

.include "m2560def.inc"
.equ SHORTVERSION=0		;1 for short, 0 for long version

main:	ldi r16, low(RAMEND)
	ldi r17, high(RAMEND)
	out SPL, r16
	out SPH, r17
	ldi r16, 0xFF
        sts DDRL, r16		;whole PORTL as output
        out DDRB, r16		;whole PORTB as output
mainloop:
;	ldi r16, 1
	ldi r16, 0x06		;Number "1" for 7 segment display on MK3 board
	out PORTB, r16		;Number 1 at PORTB
	call subroutine1
        sts PORTL, r16		;Result of subroutine1 at PORTL
	rcall wait		;wait about 1 sec
;	ldi r16, 2
	ldi r16, 0x5B		;Number "2" for 7 segment display on MK3 board
	out PORTB, r16		;Number 2 at PORTB
	call subroutine2
        sts PORTL, r16		;Result of subroutine2 at PORTL
	rcall wait		;wait about 1 sec
;	ldi r16, 3
	ldi r16, 0x4F		;Number "3" for 7 segment display on MK3 board
	out PORTB, r16		;Number 3 at PORTB
	call subroutine3
        sts PORTL, r16		;Result of subroutine3 at PORTL
	rcall wait		;wait about 1 sec
;	ldi r16, 4
	ldi r16, 0x66		;Number "4" for 7 segment display on MK3 board
	out PORTB, r16		;Number 4 at PORTB
	call subroutine4
        sts PORTL, r16		;Result of subroutine4 at PORTL
	rcall wait
	rcall wait
	rcall wait		;wait about 3 sec
        rjmp mainloop

wait:                 ; waiting subroutine
        push r24              ; save registers on stack
        push r25
        push r16
        ldi r16, 150          ; counter for outer loop
w1:     ldi r24, low(-32000)  ; r25:r24 as 16 bit counter for inner loop
        ldi r25, high(-32000)
w2:     adiw r24, 1           ; r25:r24 incrementing
        brne w2               ; branch if not zero
        dec r16
        brne w1               ; outer loop
        pop r16               ; restore registers from stack
        pop r25
        pop r24
        ret

subroutine1:			;first subroutine at low flash address
	ldi r16, (1<<0)
	ret

.if SHORTVERSION!=1
.org 0x8100			;0x10200 als Byteadresse
.endif
subroutine2:			;2. subroutine above 64KB
	ldi r16, (1<<1)
	ret
	
.if SHORTVERSION!=1
.org 0x10200			;0x20400 als Byteadresse
.endif
subroutine3:			;3. subroutine above 128KB
	ldi r16, (1<<2)
	ret

.if SHORTVERSION!=1
;.org 0x18000			;0x30000 als Byteadresse
.org 0x17F00			;0x40000 - 0x10200 = 0x2FE00 Byteadresse
.endif
subroutine5:			;5. subroutine above 192KB
	ldi r16, (1<<4)
	ret

.if SHORTVERSION!=1
.org 0x18100			;0x30200 als Byteadresse
.endif
subroutine4:			;4. subroutine above 192KB
	ldi r16, (1<<3)
	ret
